pro ca_prep, datadir, darkdir, wave = wave, flatdir = flatdir $
                  , fdarkdir = fdarkdir, savedir = savedir, fixed_ctr=fixed_ctr $
                  , sf = sf, ef = ef, data = data, par_name = par_name $
                  , time=time, no_play = no_play

;; Ca K spectroscopy Data Analysis
;; for the data obtained on 2007/04/12
;; 2007.04.12 Written  by S.UeNo
;; 2007.04.15 Modified by K.Okamoto & T.Matsumoto
;; 2007.04.19,24 Modified by T.Nakamura
;; 2007.04.27 Addition of the CAMERA option by S.UeNo
;;    U10: Apogee Alta U10, kodak: Kodak MegaPlus 1.6i
;; 2007.04.28 Addition of the Line Center option (ctr) by S.UeNo
;;    If you want to use a fixed given line center position, 
;;    you can set the X value [pix].
;;    If averaged intensity at k3 is larger than that of k1r or k1v,
;;    you SHOULD USE this option!!!
;;    ^^^^^^^^^^^^^^^^^^^^^^^^^^^
;; 2007.04.28 Modified by Otsuji
;;            option (ctr) was removed
;; 2008.02.13 Modified by Watanabe


;*****************************************************************************|
; Preparatione
;*****************************************************************************|

flag = isvalid(no_play)
flag1 = 1
if not isvalid(savedir) then savedir = '.'
if not is_dir(savedir) then file_mkdir,savedir
if not isvalid(sf) then sf = 0
if not isvalid(wave) then begin
    jump01:
    wave=''
    read,wave,prompt='%% Please Input Wavelength ca_h or ca_k. : '
endif
camera='GE1650'

datafile = findfile(datadir + '/*.{tif,fit{,s}}',count=ndata)
sss=strlen(datadir)
if n_elements(ef) eq 0 then ef = ndata-1
ndata=(ef-sf)+1

case 1 of
    max(wave eq ['ca','Ca','CA']+'_h'):begin
        wave = 'ca_h'
        tmp = read_tiff(datafile(sf))
    end
    max(wave eq ['ca','Ca','CA']+'_k'):begin
        wave = 'ca_k'
        tmp = read_tiff(datafile(sf))
    end
    max(wave eq ['ca','Ca','CA']+'_H'):begin
        wave = 'ca_h'
        tmp = read_tiff(datafile(sf))
    end
    max(wave eq ['ca','Ca','CA']+'_K'):begin
        wave = 'ca_k'
        tmp = read_tiff(datafile(sf))
    end
    max(wave eq ['H','h']):begin
        wave = 'ca_h'
        tmp = read_tiff(datafile(sf))
    end
    max(wave eq ['K','k']):begin
        wave = 'ca_k'
        tmp = read_tiff(datafile(sf))
    end
    else:begin
        print,'%% There is no wavelength format named "'+wave+'".'
        goto,jump01
    end
endcase

time = fn2time(datafile(sf:ef),camera=camera)
ss   = size(tmp,/dim)

;;;;;;;;;;;;;;;
sslt = 0             ; 3    ; $B%9%j%C%H$H%9%-%c%sJ}8~$,?bD>$GL5$$>l9g!"(B
                            ; $B#1%9%F%C%W;v$K6u4VJ}8~2?%T%/%;%k$:$D$:$i$9$+!#(B
;;;;;;;;;;;;;;;

;*****************************************************************************|
; Dark 
;*****************************************************************************|

dgenx=file_search(darkdir+'/dark.genx',count=dd)

if darkdir eq 'N/A' then dd=-1

case dd of
    -1:begin
        print,'%% Making assumed dark file.'
        dark = fltarr(ss[0],ss[1]) 
    end
    0:begin
        print,'%% Makeing dark file.'
        
        darkfile = findfile(darkdir+'/*.{tif,fit{,s}}',count=ndark) ; $B%@!<%/%U%l!<%`$N%U%)%k%@L>(B
        dark     = fltarr(ss[0],ss[1])
        
        
        for n = 0, ndark-1 do dark = temporary(dark) + read_tiff(darkfile(n))/16.
        
        dark     = temporary(dark)/ndark
        darkFLUX = total(dark(3:ss[0]-3,70:ss[1]-70))*0.000451

        savegen,file=darkdir+'/dark.genx',dark
    end
    else:begin
        print,'%% Reading dark.genx.'
        restgen,file=dgenx,dark
    end
endcase

;*****************************************************************************|
; Dark for flat
;*****************************************************************************|

fgenx=file_search(flatdir+'/flat.genx',count=ff)
fdgenx=file_search(fdarkdir+'/dark.genx',count=fd)
if fdarkdir eq 'N/A' then fd=-1
if ff ne 0 then fd=-2

case fd of
    -2:                         ;for skipping
    -1:begin
        print,'%% Making assumed dark file for flat.'
        fdark = fltarr(ss[0],ss[1]) 
    end
    0:begin
        print,'%% Makeing dark file for flat.'
        
        fdarkfile = findfile(fdarkdir+'/*.{tif,fit{,s}}',count=nfdark) ; $B%@!<%/%U%l!<%`$N%U%)%k%@L>(B
        fdark     = fltarr(ss[0],ss[1])
        
        for n = 0, nfdark-1 do fdark = temporary(fdark) + read_tiff(fdarkfile(n))/16.

        
        fdark     = temporary(fdark)/nfdark

        savegen,file=fdarkdir+'/dark.genx',fdark
    end
    else:begin
        print,'%% Reading dark.genx for flat.'
        restgen,file=fdgenx,fdark
    end
endcase

;*****************************************************************************|
; Flat
;*****************************************************************************|

case isvalid(flatdir) of
    0:begin
        print,'%% Makeing assumed flat file.'
        flat = replicate(1.,ss[0],ss[1])
    end
    1:begin
        if flatdir eq 'N/A' then ff=-1
        case ff of
            -1:begin
                print,'%% Makeing assumed flat file.'
                flat = replicate(1.,ss[0],ss[1])
            end
            0:begin
                print,'%% Makeing flat file.'
                flat = fltarr(ss[0],ss[1])
                flatfile=file_search(flatdir+'/*.{tif,fit{,s}}',count=nflat) ; $B%U%i%C%H%U%l!<%`$N%U%)%k%@L>(B
                
                for i = 0, nflat-1 do flat = temporary(flat) + read_tiff(flatfile(i))/16.        
                
                flat = temporary(flat)/nflat-fdark
                tmpprof1=total(flat,2)
                tmpprof2=smooth(total(flat,2),21)
                w2=where(tmpprof2 eq min(tmpprof2))
                w1=where(tmpprof1(w2(0)-20:w2(0)+20) eq min(tmpprof1(w2(0)-20:w2(0)+20)))+(w2(0)-20)

                lc1=fltarr(401)
                lc2=fltarr(401)
                for y=100,500 do begin
                    lc1(y-100)=where( flat(w1(0)-10:w1(0)+10,y) eq min(flat(w1(0)-10:w1(0)+10,y)) )+(w1(0)-10.)
                    c=poly_fit(dindgen(13),flat(lc1(y-100)-6:lc1(y-100)+6,y),2)
                    lc2(y-100)=-1.*c(1)/2./c(2) + (lc1(y-100)-6.)
                    if (lc2(y-100) lt lc1(y-100)-10.) or (lc2(y-100) gt lc1(y-100)+10.) $
                      then begin lc2(y-100) = lc1(y-100)
                    endif
                endfor

                c=poly_fit(indgen(401),lc2,1)
                tilt= 180./!pi*atan(c(1)) ; [deg]

                flat2=rot(flat,-tilt,/interp)

                profx=total(flat2,2)
                profy=total(flat2,1)
                meanimg=profx#profy

                flat=flat2/meanimg
                flat=flat/mean(flat)
                flat=rot(flat,tilt,/interp)
                
                savegen,file=flatdir+'/flat.genx',flat
            end
            else:begin
                print,'%% Reading flat.genx.'
                restgen,file=fgenx,flat
            end
        endcase
    end
endcase

;*****************************************************************************|
; Array definition
;*****************************************************************************|

ny   = ss[1] + sslt*ndata


;*****************************************************************************|
; Plot Start
;*****************************************************************************|

;if flag ne 1 then begin
;    window, 0, xs= 400, ys = 400, xpos =   0, ypos = 0
;    window, 1, xs= 400, ys = 400, xpos = 400, ypos = 0
;    window, 2, xs= 400, ys = 400, xpos = 800, ypos = 0
;endif

for n = sf, ef do begin
    
    if (n mod 10 eq 0) then print,strtrim(n,2)+'/'+strtrim(ef,2)
    
        a = read_tiff(datafile(n))

    
  ;  if mean(a)/mean(dark) lt 1.1 then goto,tobasu2

    a = (temporary(a) - dark)*1./flat
    a = [rebin(a(5,*),5,ss(1)),a(5:ss(0)-6,*),rebin(a(ss(0)-6,*),5,ss(1))]

;*** Rotation Compensation$B!J%9%Z%/%H%k2hA|$,%+%a%i$KBP$7$F79$$$F$$$?>l9g!K(B
;*** calibration ***

    if flag1 then begin
        y1 = ss(1)/4
        y2 = ss(1)/4*3-1
        ll = y2-y1+1
        w  = (cos(!pi*findgen(ss(0))/ss(0))^2+1)/2

        if(wave eq 'ca_k') then begin
        print,'%% Reading ./cak_refspec_'+camera+'.genx.'
        restgen,file='./cak_refspec_'+camera+'.genx',prof0,ctr0
        endif
        if(wave eq 'ca_h') then begin
        print,'%% Reading ./cah_refspec_'+camera+'.genx.'
        restgen,file='./cah_refspec_'+camera+'.genx',prof0,ctr0
        endif
       
	prof0=rebin(temporary(prof0)*w,ss(0),ll)
        prof0=reform([prof0(0,*),prof0],n_elements(prof0)+ll)
        prof0=reform(prof0(ll/2:n_elements(prof0)-ll/2-1),ss(0),ll)

        prof1 = rebin(smooth(a(*,y1),2)*w,ss(0),ll)

        cor1=max(total(prof0*prof1,1)/sqrt(total(prof0^2,1))/sqrt(total(prof1^2,1)))
        cor2=max(total(rotate(prof0,2)*prof1,1)/sqrt(total(prof0^2,1))/sqrt(total(prof1^2,1)))

	case 1 of
	    cor1 ge cor2:print,'%% Profile direction correct.'
	    else:begin
		print,'%% Profile direction reversed.'
		prof0 = rotate(temporary(prof0),2)
		ctr0  = ss(0) - ctr0 - 2
	    end
	endcase
	flag1 = 0
    endif

    prof1 = rebin(smooth(a(*,y1),2)*w,ss(0),ll)
    prof2 = rebin(smooth(a(*,y2),2)*w,ss(0),ll)
    cor1=max(total(prof0*prof1,1)/sqrt(total(prof0^2,1))/sqrt(total(prof1^2,1)),dx1)
    cor2=max(total(prof0*prof2,1)/sqrt(total(prof0^2,1))/sqrt(total(prof2^2,1)),dx2)

    angle=-atan(dx2-dx1,ll-1)*!radeg
    ctr1=dx1-(ll/2-1)+ctr0
    ctr2=dx2-(ll/2-1)+ctr0
    roll_xy,ctr1,y1,angle,ctr,center=ss/2-1
    ctr=round(ctr)

    a = rot(temporary(a),angle,/interp)
    ssss=strlen(datafile(n))
    savegenx,a,file=savedir+'/'+strmid(datafile(n),sss+1,ssss-sss-5)
    
endfor

ended:
end
