function fn2time,filename,filtergram=filtergram,yyyy=yyyy,camera=camera
;+
;  PURPOSE  : convert DST output filename to time [string]
;
;  SYNTAX   : time = fn2time(filename)
;
;  INPUT    : filename
;
;  OPTION   : yyyy -> input yyyy for HIS
;  
;  KEYWORD  : /filtergram -> if 'filename' is file name for HIS
;
;  HISTORY  : 2007.04.16 Written by T.Matsumoto
;           : 2007.04.27 Addition of CAMERA option by S.UeNo
;                U10: Apogee Alta U10,  kodak: Kodak MegaPlus1.6i
;           : 2007.04.30 Simplified by K.Otsuji
;           : 2007.05.28 Addition of new CAMERA by S.UeNo
;                GE1650: Prosilica GE1650
;-

nn   = n_elements(filename)
time = strarr(nn)

month = ['Jan','Feb','Mar','Apr','May','Jun' $
         ,'Jul','Aug','Sep','Oct','Nov','Dec']

;*****************************************************************************|
; For Hida Imaging System (HIS)
;*****************************************************************************|

if keyword_set(filtergram) then begin
    temp=strmid(filename,[27,25,23,21,19],[2,2,2,2,2],/r)
    if not isvalid(yyyy) then yyyy='2007'
    yyyy=replicate(yyyy,1,nn)
    time=yyyy+'-'+temp(0,*)+'-'+temp(1,*) $
      +' '+temp(2,*)+':'+temp(3,*)+':'+temp(4,*)
    return,time
endif

;*****************************************************************************|
; For Vertical Spectrograph (VS)
;*****************************************************************************|

if max(camera eq ['KODAK','kodak']) eq 1 then begin
    temp=strmid(filename,[18,14,12,9,7,5],[4,2,2,2,2,2],/r)
    time=temp(0,*)+'-'+temp(1,*)+'-'+temp(2,*) $
      +' '+temp(3,*)+':'+temp(4,*)+':'+temp(5,*)
endif
if max(camera eq ['u','U']+'10') eq 1 then begin
    mreadfits,filename,h,/silent,/nodata
    time=h.DATE_D$OBS+' '+h.TIME_D$OBS
endif
if max(camera eq ['GE1650','ge1650']) eq 1 then begin
    temp=strmid(filename,[26,21,18,15,12,9,6],[4,2,2,2,2,2,3],/r)
    time=temp(0,*)+'-'+temp(1,*)+'-'+temp(2,*) $
      +' '+temp(3,*)+':'+temp(4,*)+':'+temp(5,*)+'.'+temp(6,*)
endif

return,time

end
